// --------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
// --------------------------------------------------------------------
// Module: Uart_bus
// 
// Author: Step
// 
// Description: The module for uart communication
// 
// Web: www.stepfapga.com
// 
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Version: |Mod. Date:   |Changes Made:
// V1.0     |2016/04/20   |Initial ver
// --------------------------------------------------------------------
module Uart_Bus #
(
parameter				BPS_PARA = 104 // 1250 for 9600 baud rate when 12MHz clk_in;104 for 115200 baudrate
)
(
input					clk_in,
input					rst_n_in,
input					rs232_rx,
output					rs232_tx,
output                  uart_tx_valid
);		
	
/////////////////////////////////uart_rx module////////////////////////////////////
wire					bps_en_rx,bps_clk_rx;
wire			[7:0]	rx_data;

Baud #
(
.BPS_PARA				(BPS_PARA		)
)
Baud_rx
(	
.clk_in					(clk_in			),	// 12MHz system clock
.rst_n_in				(rst_n_in		),	// system reset, active low
.bps_en					(bps_en_rx		),	// start beat request
.bps_clk				(bps_clk_rx		)	// data receive beat for 9600 bps
);

Uart_Rx Uart_Rx_uut
(
.clk_in					(clk_in			),	// 12MHz system clock
.rst_n_in				(rst_n_in		),	// system reset, active low
.bps_en					(bps_en_rx		),	// beat clock enable
.bps_clk				(bps_clk_rx		),	// beat clock input
.rs232_rx				(rs232_rx		),	// uart receive pin
.rx_data				(rx_data		)	// data of uart receive
);
	

/////////////////////////////////uart_tx module////////////////////////////////////
wire					bps_en_tx,bps_clk_tx,uart_tx_valid;

Baud #
(
.BPS_PARA				(BPS_PARA		)
)
Baud_tx
(
.clk_in					(clk_in			),	// 12MHz system clock
.rst_n_in				(rst_n_in		),	// system reset, active low
.bps_en					(bps_en_tx		),	// start beat request
.bps_clk				(bps_clk_tx		)	// data receive beat for 9600 bps
);

Uart_Tx Uart_Tx_uut
(
.clk_in					(clk_in			),	// 12MHz system clock
.rst_n_in				(rst_n_in		),	// system reset, active low
.bps_en					(bps_en_tx		),	// beat clock enable
.bps_clk				(bps_clk_tx		),	// beat clock input
.rx_bps_en				(bps_en_rx		),	// 
.tx_data				(rx_data		),	// data need to transfer
.rs232_tx				(rs232_tx		),	// uart transfer pin
.uart_tx_valid 			(uart_tx_valid	)	// 1:indicate tx done ,0:transfering,can't send new data to it 
);

endmodule
