// --------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<
// --------------------------------------------------------------------
// Module: Uart_Tx
// 
// Author: Step
// 
// Description: The transfer module of uart interface
// 
// Web: www.stepfapga.com
// 
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Version: |Mod. Date:   |Changes Made:
// V1.0     |2016/04/20   |Initial ver
// --------------------------------------------------------------------
module Uart_Tx
(
input					clk_in,			// 12MHz system clock
input					rst_n_in,		// system reset, active low

output	reg				bps_en,			// beat clock enable
input					bps_clk,		// beat clock input
input                   uart_tx_en,

input					rx_bps_en,		// 
input			[7:0]	tx_data,		// data need to transfer
output					rs232_tx,		// uart transfer pin
output 	reg				uart_tx_valid
);

reg						rx_bps_en_r;
reg 					rs232_tx_r;
//Detect negedge of rx_int
always @ (posedge clk_in or negedge rst_n_in) begin
	if(!rst_n_in) rx_bps_en_r <= 1'b0;
	else rx_bps_en_r <= rx_bps_en;
end
	
wire	neg_rx_bps_en ;
//assign neg_rx_bps_en = uart_tx_en ? rx_bps_en_r & (~rx_bps_en) : 0 ;
assign neg_rx_bps_en = uart_tx_en ? 1 : 0 ;
assign rs232_tx =  rs232_tx_r ;

reg				[3:0]	num;
reg				[9:0]	tx_data_r;	

//start 8bit_data transfer operation
always @ (posedge clk_in or negedge rst_n_in) begin
	if(!rst_n_in) begin
		bps_en <= 1'b0;
		tx_data_r <= 8'd0;

	end else if(neg_rx_bps_en)begin	
		bps_en <= 1'b1;
		tx_data_r <= {1'b1,tx_data,1'b0};	

	end else if(num==4'd10) begin	
		bps_en <= 1'b0;

	end
end

//transfer data with bps_clk when bps_en is actived
always @ (posedge clk_in or negedge rst_n_in) begin
	if(!rst_n_in) begin
		num <= 1'b0;
		rs232_tx_r <= 1'b1;
	end else if(bps_en&&uart_tx_en) begin  
         
		if(bps_clk) begin
			num <= num + 1'b1;
			rs232_tx_r <= tx_data_r[num];
           		
		end else if(num>=4'd10) begin
			num <= 4'd0;
			end
        
	end
end

always @(posedge clk_in or negedge rst_n_in) 
begin
    	if(!rst_n_in) begin    
		uart_tx_valid <= 1'b0;
        end else begin 
             if(uart_tx_en&&num<4'd10) uart_tx_valid <= 1'b1;
             if(num>=4'd10) uart_tx_valid <= 1'b0;
        end
end

endmodule
