module compute(
input clk_in,
input rst_n_in,
input uart_tx_valid,
input  [8:0]    key_operate_pulse,
input  [10:0]   key_num_pulse,
output reg uart_tx_en,
output reg [7:0]  uart_data
);

parameter IDLE        = 4'd0,
          INPUT       = 4'd1,
          OPERATE1    = 4'd2,
          OPERATE2    = 4'd3,
          CLEAR       = 4'd4,
          EQUAL       = 4'd5,         
          BCDSTART    = 4'd6,
          BCDCONVERT  = 4'd7,
          BCDSEND     = 4'd8,
          ENDSTA      = 4'd9;
                           
reg [3:0] state;
reg [15:0] value1,value2,cal_result,value_result = 0;
reg [3:0] cal_number,cal_mode,operate_reg = 0;
reg [2:0] cal_num_count = 0; 
reg [3:0] cnt_send;
reg [3:0]cnt_convert=0;
reg equal_flag,operate2_flag,equal_data_flag,uart_send_done;

reg [15:0] bcd_out_data;
wire [15:0] bcd_out;
reg [7:0] bcd_data_reg [4:0];
reg [7:0] cal_data_reg;
reg bin_to_bcd_start;
always@(posedge clk_in or negedge rst_n_in) 
begin
	if(!rst_n_in) begin
        cal_number <= 0;
        cal_mode <= 0;
        operate_reg <= 0;
        cal_num_count <= 0; 
        value1<=0;
        value2<=0;
        cal_result <= 0;
        value_result <= 0;
        equal_flag <= 0;
		operate2_flag <= 0;
        state <= IDLE;       
        cnt_convert <= 0;
        equal_data_flag <= 0;
        bin_to_bcd_start <= 1'b0;
	end else begin
        case(state)
        IDLE:begin
            cal_number <= 0;
            cal_mode <= 0;
            operate_reg <= 0;
            cal_num_count <= 0; 
            value1<=0;
            value2<=0;
            cal_result <= 0;
            value_result <= 0;
            equal_flag <= 0;
            operate2_flag <= 0;
            state <= INPUT;       
            cnt_convert <= 0;
            equal_data_flag <= 0;
            bin_to_bcd_start <= 1'b0;
        end   
    INPUT:begin
                if(key_num_pulse) begin
                case(key_num_pulse)
                    11'b000_0000_0001:begin cal_number <= 4'd0; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//0
                    11'b000_0000_0010:begin cal_number <= 4'd1; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//1
                    11'b000_0000_0100:begin cal_number <= 4'd2; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//2
                    11'b000_0000_1000:begin cal_number <= 4'd3; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//3
                    11'b000_0001_0000:begin cal_number <= 4'd4; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//4
                    11'b000_0010_0000:begin cal_number <= 4'd5; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//5
                    11'b000_0100_0000:begin cal_number <= 4'd6; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//6
                    11'b000_1000_0000:begin cal_number <= 4'd7; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//7
                    11'b001_0000_0000:begin cal_number <= 4'd8; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//8
                    11'b010_0000_0000:begin cal_number <= 4'd9; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//9
                    11'b100_0000_0000:begin cal_number <= 4'd0; if(cal_num_count <= 5) cal_num_count <= cal_num_count + 1'b1;else cal_num_count <= cal_num_count;state <= OPERATE1;  end//.
                default:;
                endcase               
                end  
                else if(key_operate_pulse) begin
                case(key_operate_pulse)              
                    9'b0_0000_0001:begin  state <= CLEAR;end//C
                    9'b0_0000_0010:begin  state <= EQUAL;end//=
                    9'b0_0000_0100:begin cal_mode <= 4'd3; state <= OPERATE2;end//+
                    9'b0_0000_1000:begin cal_mode <= 4'd4; state <= OPERATE2;end//-
                    9'b0_0001_0000:begin cal_mode <= 4'd5; state <= OPERATE2;end//*
                    9'b0_0010_0000:begin cal_mode <= 4'd6; state <= OPERATE2;end///
                    9'b0_0100_0000:begin state <= OPERATE2;end//%
                    9'b0_1000_0000:begin state <= OPERATE1;end//-+
                    9'b1_0000_0000:begin state <= IDLE;end//AC
                default:cal_mode <= 0;
                endcase
            end else begin
                state <= INPUT;
            end 
        end
    OPERATE1:begin
            if(operate2_flag) begin
                value2 <= value2*10 + cal_number; 
            end else begin
                value1 <= value1*10 + cal_number; 
            end
            state <= INPUT;
        end
    OPERATE2:begin
            operate2_flag <= 1'b1;
            operate_reg <= cal_mode;
            state <= INPUT;   
        end
    CLEAR:begin
                cal_number <= 0;
                cal_mode <= 0;
                cal_num_count <= 0; 
                value1 <= 0;
                value2 <= 0;
                cal_result <= 0;
                equal_flag <= 0;
                operate2_flag <= 0;           
                cnt_convert <= 0;                
                equal_data_flag <= 0;
                bin_to_bcd_start <= 1'b0;
                state <= IDLE;
        end
    EQUAL:begin
            case(operate_reg)
            4'd3:begin cal_result <= value1 + value2;end
            4'd4:begin cal_result <= value1 - value2;end
            4'd5:begin cal_result <= value1 * value2;end
            4'd6:begin cal_result <= value1 / value2;end          
            default:cal_result <=0;
            endcase
            operate2_flag <= 0;
            value1 <= 0;
            value2 <= 0;
            state <= BCDSTART;
            bin_to_bcd_start <= 1'b1;
        end

    BCDSTART:begin 
            bcd_out_data <= bcd_out;
            
            /*
            value_result[15:12] <= cal_result/1000;
            value_result[11:8] <= (cal_result%1000)/100;
            value_result[7:4] <= (cal_result%100)/10;
            value_result[3:0] <= cal_result%10;
            */
            state <= BCDCONVERT;   
                    
        end
    BCDCONVERT:begin	
            
            if(cnt_convert>= 4) begin 
                cnt_convert <= 0; state <= BCDSEND; 
            end else begin
                cnt_convert <= cnt_convert + 1'b1; 
                bcd_out_data <= bcd_out_data << 4;                  
                state <= BCDCONVERT;
                
                case(bcd_out_data[15:12])
                    4'd0: bcd_data_reg[cnt_convert] <= "0";  
                    4'd1: bcd_data_reg[cnt_convert] <= "1"; 
                    4'd2: bcd_data_reg[cnt_convert] <= "2"; 
                    4'd3: bcd_data_reg[cnt_convert] <= "3"; 
                    4'd4: bcd_data_reg[cnt_convert] <= "4"; 
                    4'd5: bcd_data_reg[cnt_convert] <= "5"; 
                    4'd6: bcd_data_reg[cnt_convert] <= "6"; 
                    4'd7: bcd_data_reg[cnt_convert] <= "7"; 
                    4'd8: bcd_data_reg[cnt_convert] <= "8"; 
                    4'd9: bcd_data_reg[cnt_convert] <= "9"; 
                default:;
            endcase  
                             
            end            
                
          end
    BCDSEND:begin
                equal_data_flag <= 1; 
                state <= ENDSTA;
            end
        
    ENDSTA:begin
            if(uart_send_done) begin
                cal_number <= 0;
                cal_mode <= 0;
                cal_num_count <= 0; 
                value1 <= 0;
                value2 <= 0;
                cal_result <= 0;
                equal_flag <= 0;
                operate2_flag <= 0;           
                cnt_convert <= 0;                
                equal_data_flag <= 0;
                bin_to_bcd_start <= 1'b0;
                state <= IDLE;
            end else begin
                state <= ENDSTA;
            end

        end
        default:;
        endcase
    end    
end

bin_bcd #(.B_SIZE(12))bin_bcd_u(
.rst_n(bin_to_bcd_start),
.binary(cal_result),
.bcd(bcd_out)
); 
reg[7:0] data_out_reg =0;
reg uart_tx_en;
reg [2:0] txstate;
reg [15:0] cnt_delay;
parameter   SENDIDLE    = 3'd0,
            DELAY       = 3'd1,
            SEND        = 3'd2,  
            EQUALSTA    = 3'd3,
            EQUALDELAY  = 3'd4,
            EQUALSEND   = 3'd5;

always@(posedge clk_in or negedge rst_n_in) begin
	if(!rst_n_in) begin
        data_out_reg <=0;
        txstate <= SENDIDLE; 
        uart_tx_en <= 0;
        uart_send_done <= 0; 
        cnt_send <= 0;
	end else begin
        case(txstate)
        SENDIDLE: begin
            if(key_num_pulse) begin
                    case(key_num_pulse)
                        11'b000_0000_0001:begin uart_tx_en <= 1;data_out_reg <= "0";txstate <= DELAY; end//0
                        11'b000_0000_0010:begin uart_tx_en <= 1;data_out_reg <= "1";txstate <= DELAY; end//1
                        11'b000_0000_0100:begin uart_tx_en <= 1;data_out_reg <= "2";txstate <= DELAY; end//2
                        11'b000_0000_1000:begin uart_tx_en <= 1;data_out_reg <= "3";txstate <= DELAY; end//3
                        11'b000_0001_0000:begin uart_tx_en <= 1;data_out_reg <= "4";txstate <= DELAY; end//4
                        11'b000_0010_0000:begin uart_tx_en <= 1;data_out_reg <= "5";txstate <= DELAY; end//5
                        11'b000_0100_0000:begin uart_tx_en <= 1;data_out_reg <= "6";txstate <= DELAY; end//6
                        11'b000_1000_0000:begin uart_tx_en <= 1;data_out_reg <= "7";txstate <= DELAY; end//7
                        11'b001_0000_0000:begin uart_tx_en <= 1;data_out_reg <= "8";txstate <= DELAY; end//8
                        11'b010_0000_0000:begin uart_tx_en <= 1;data_out_reg <= "9";txstate <= DELAY; end//9
                        11'b100_0000_0000:begin uart_tx_en <= 1;data_out_reg <= ".";txstate <= DELAY; end//.
                    default:begin uart_tx_en <= 0;data_out_reg <= 0; txstate <= SENDIDLE;end 
                    endcase  
                     
            end else if(key_operate_pulse) begin
                case(key_operate_pulse)              
                    9'b0_0000_0001:begin uart_tx_en <= 1;data_out_reg <= "C";txstate <= DELAY;end//C
                    9'b0_0000_0010:begin uart_tx_en <= 1;data_out_reg <= "=";txstate <= DELAY;end//=
                    9'b0_0000_0100:begin uart_tx_en <= 1;data_out_reg <= "+";txstate <= DELAY;end//+
                    9'b0_0000_1000:begin uart_tx_en <= 1;data_out_reg <= "-";txstate <= DELAY;end//-
                    9'b0_0001_0000:begin uart_tx_en <= 1;data_out_reg <= "*";txstate <= DELAY;end//*
                    9'b0_0010_0000:begin uart_tx_en <= 1;data_out_reg <= "/";txstate <= DELAY;end///
                    9'b0_0100_0000:begin uart_tx_en <= 1;data_out_reg <= "%";txstate <= DELAY;end//%
                    9'b0_1000_0000:begin uart_tx_en <= 1;data_out_reg <= "-";txstate <= DELAY;end//-+
                    9'b1_0000_0000:begin uart_tx_en <= 1;data_out_reg <= "A";txstate <= DELAY;end//AC
                default:begin uart_tx_en <= 0;data_out_reg <= 0; txstate <= SENDIDLE;end 
                endcase                  
            end 
            else begin
                uart_tx_en <= 0;data_out_reg <= 0; txstate <= SENDIDLE;
            end        
          end  
        DELAY:begin 
                
                if(cnt_delay>=3) begin cnt_delay <= 0; uart_data <= data_out_reg; txstate <= SEND;end                
                else begin cnt_delay <= cnt_delay + 1'b1; txstate <= DELAY; end
                
            end
        SEND:begin
                if(uart_tx_valid) begin 
                    uart_tx_en <= 1;txstate <= SEND; 
                end else begin 
                    uart_tx_en <= 0; 
                    if(data_out_reg == "=")begin 
                        txstate <= EQUALSTA;  
                    end else begin
                        txstate <= SENDIDLE;
                    end
                end
            end
            
        EQUALSTA:begin
                if(equal_data_flag) begin                    
                    txstate <= EQUALSEND;                     
                    if(cnt_send >= 5) begin 
                        cnt_send <= 0; 
                        txstate <= SENDIDLE; 
                        uart_send_done <= 1'b1;                               
                    end else if(cnt_send == 4) begin
                        uart_data <= "$";
                        cnt_send <= cnt_send + 1'b1;
                    end else begin 
                        uart_data <= bcd_data_reg[cnt_send];
                        cnt_send <= cnt_send + 1'b1;
                    end                   
                end else begin                    
                    txstate <= EQUALSTA;
                    uart_send_done <= 1'b0; 
                end 
                

            end
        EQUALDELAY:begin                
                if(cnt_delay>=1200) begin cnt_delay <= 0;  txstate <= EQUALSTA; end                
                else begin cnt_delay <= cnt_delay + 1'b1; txstate <= EQUALDELAY; end              
            end
        EQUALSEND:begin
                if(uart_tx_valid) begin uart_tx_en <= 1;txstate <= EQUALSEND; end
                else begin uart_tx_en <= 0;txstate <= EQUALDELAY;   end                  
            end                                   
        default:txstate <= SENDIDLE; 
        endcase           
    end    
end 

endmodule
