#include "LCD_Driver.h"
#include "MicoUtils.h"
#include "MicoGPIO.h"
#include "DDStructs.h"

extern MicoGPIOCtx_t *lcd_gpio;
extern MicoGPIOCtx_t *lcd_sda;
extern MicoGPIOCtx_t *lcd_sck;

/**
* 	3	2	1	0
*  		RS	RST	CS
*/
#define LCD_CS_L	MICO_GPIO_WRITE_DATA_BYTE0(lcd_gpio->base,0x02)//0010
#define LCD_CS_H	MICO_GPIO_WRITE_DATA_BYTE0(lcd_gpio->base,0x03)//0011

#define LCD_RST_L	MICO_GPIO_WRITE_DATA_BYTE0(lcd_gpio->base,0x00)//0000
#define LCD_RST_H	MICO_GPIO_WRITE_DATA_BYTE0(lcd_gpio->base,0x02)//0010

#define LCD_RS_L	MICO_GPIO_WRITE_DATA_BYTE0(lcd_gpio->base,0x02)//0010
#define LCD_RS_H	MICO_GPIO_WRITE_DATA_BYTE0(lcd_gpio->base,0x06)//0110
/**
* 	3	2	1	0
*  				sda
*/
#define LCD_SDA_L	MICO_GPIO_WRITE_DATA_BYTE0(lcd_sda->base,0)//0000
#define LCD_SDA_H	MICO_GPIO_WRITE_DATA_BYTE0(lcd_sda->base,1)//0001
/**
* 	3	2	1	0
*  				sCK
*/
#define LCD_SCK_L	MICO_GPIO_WRITE_DATA_BYTE0(lcd_sck->base,0)//0000
#define LCD_SCK_H	MICO_GPIO_WRITE_DATA_BYTE0(lcd_sck->base,1)//0001


static void delay(unsigned int delaynum)
{
	MicoSleepMilliSecs(delaynum);
}

static void delay_us(unsigned int delay_usnum)
{
	
	MicoSleepMicroSecs(delay_usnum);
}


/*дָLCD ģ*/
static void transfer_command_lcd(char data)
{
	unsigned char i;
	LCD_CS_L;
	LCD_RS_L;
	for(i=0;i<8;i++)
	{
		LCD_SCK_L;
		delay_us(1);
		if(data&0x80) LCD_SDA_H;
		else LCD_SDA_L;
		LCD_SCK_H;
		delay_us(1);
		data=data<<1;
	}
	LCD_CS_H;
}
/*дݵLCD ģ*/
static void transfer_data_lcd(char data)
{
	unsigned char i;
	LCD_CS_L;
	LCD_RS_H;
	for(i=0;i<8;i++)
	{
	LCD_SCK_L;
	if(data&0x80) LCD_SDA_H;
	else LCD_SDA_L;
	LCD_SCK_H;
	data=data<<1;
	}
	LCD_CS_H;
}



static void lcd_address(unsigned char page,unsigned char column)
{
	column=column;
	transfer_command_lcd(0xb0+page); /*ҳַ*/
	transfer_command_lcd(0x10+(column>>4&0x0f)); /*еַĸ4 λ*/
	transfer_command_lcd(column&0x0f); /*еַĵ4 λ*/
}

/*ʾ8x16 ͼASCII, 8x16 ַͼ*/
void display_graphic_8x16(unsigned char page,unsigned char column,unsigned char *dp)
{
	unsigned char i,j;
	for(j=0;j<2;j++)
	{
		lcd_address(page+j,column);
		for (i=0;i<8;i++)
		{
			transfer_data_lcd(*dp); /*дݵLCD,ÿдһ8 λݺ
			еַԶ1*/
			dp++;
		}
	}
}

void initial_lcd(void)
{
	LCD_RST_L; //Reset the chip when reset=0
	delay(20);
	LCD_RST_H;
	transfer_command_lcd(0xe2); /*λ*/
	transfer_command_lcd(0x2c); /*ѹ1*/
	delay(5);
	transfer_command_lcd(0x2e); /*ѹ2*/
	delay(5);
	transfer_command_lcd(0x2f); /*ѹ3*/
	delay(5);
	transfer_command_lcd(0x20); /*ֵԱȶȣ÷Χ2027*/
	transfer_command_lcd(0x81); /*΢Աȶ*/
	transfer_command_lcd(0x15); /*΢Աȶȵֵ÷Χ063*/
	transfer_command_lcd(0xa3); /*1/9 ƫѹȣbias*/
	transfer_command_lcd(0xc0); /*ɨ˳򣺴ϵ*/
	transfer_command_lcd(0xa1); /*ɨ˳򣺴*/
	transfer_command_lcd(0x40); /*ʼУӵһпʼ*/
	transfer_command_lcd(0xa4); 
	transfer_command_lcd(0xaf); /*ʾ*/
}

void clear_screen(void)
{
	unsigned char i,j;
	for(i=0;i<4;i++)
	{
		transfer_command_lcd(0xb0+i);
		transfer_command_lcd(0x10);
		transfer_command_lcd(0x00);
		for(j=0;j<132;j++)
		{
		transfer_data_lcd(0x00);
		}
	}
}
