/**************************************************
module: lcdDriver
author: wanganran
description: LCDʾ1
input: clk,rst_n
output: lcd_rst_n_out,lcd_cs_n_out,lcd_dc_outlcd_clk_outlcd_data_out
date: 2015.10.23
**************************************************/
module calculator_lcd
(
input clk,  //clk = 12mhz
input rst_n,  //rst_n, active low
output reg lcd_rst_n_out,  // reset, active low
output reg lcd_cs_n_out,  // chip select, active low
output reg lcd_dc_out,  // data or command control
output lcd_clk_out,  // clock
output reg lcd_data_out  // data
);

parameter CLK_DIV_PERIOD=10; //related with clk_div's frequency
parameter DELAY_PERIOD=12000;  //related with delay time and refresh frequency

parameter CLK_L=2'd0;
parameter CLK_H=2'd1;
parameter CLK_RISING_DEGE=2'd2;
parameter CLK_FALLING_DEGE=2'd3;

parameter IDLE=3'd0;
parameter SHIFT=3'd1;
parameter CLEAR=3'd2;
parameter SETCMD=3'd3;
parameter DISPLAY=3'd4;
parameter DELAY=3'd5;

parameter LOW =1'b0;
parameter HIGH =1'b1;
parameter CMD =1'b0;
parameter DATA =1'b1;

//assign lcd_rst_n_out = 1;  //active low level, set 1 for normal
//assign lcd_dc_out = 1;

//initial for memory register
reg [7:0] cmd_r [12:0];
initial
	begin
		//initial
		cmd_r[0]= 8'he2;   // command for initial
		cmd_r[1]= 8'h2c;   // command for initial
		cmd_r[2]= 8'h2e;   // command for initial
		cmd_r[3]= 8'h2f;   // command for initial
		cmd_r[4]= 8'h20;   // command for initial
		cmd_r[5]= 8'h81;   // command for set row1
		cmd_r[6]= 8'h15;   // command for set row2
		cmd_r[7]= 8'ha3;   // command for set row3
		cmd_r[8]= 8'hc0;   // command for set row4
		cmd_r[9]= 8'ha1;
	    cmd_r[10]= 8'h40;
        cmd_r[11]= 8'ha4;
		cmd_r[12]= 8'haf;
		//clear screen
		cmd_r[13]= 8'hb0;
		cmd_r[14]= 8'hb1;
		cmd_r[15]= 8'hb2;
		cmd_r[16]= 8'hb3;
		cmd_r[17]= 8'h10;
		cmd_r[18]= 8'h00;
	end
//initiacmd_r[8]= 8'hc0;l for memory register
reg [63:0] mem [27:0];
reg [63:0] temp;
initial
	begin
		mem[0] = {8'h00, 8'h20, 8'h30, 8'h10, 8'hF8, 8'h00, 8'h00, 8'h00}; 
		mem[1] = {8'h00, 8'h00, 8'h00, 8'h00, 8'h1F, 8'h00, 8'h00, 8'h00};   //1
		mem[2] = {8'h20, 8'h10, 8'h08, 8'h08, 8'h08, 8'h08, 8'h90, 8'h70};
		mem[3] = {8'h10, 8'h18, 8'h14, 8'h12, 8'h12, 8'h11, 8'h10, 8'h10};   //2
		mem[4] = {8'h10, 8'h08, 8'h08, 8'h88, 8'h88, 8'h88, 8'hD0, 8'h70}; 
		mem[5] = {8'h08, 8'h10, 8'h10, 8'h10, 8'h10, 8'h10, 8'h09, 8'h07};   //3
		mem[6] = {8'h00, 8'h00, 8'h80, 8'h40, 8'h20, 8'hF8, 8'h00, 8'h00};
		mem[7] = {8'h06, 8'h05, 8'h04, 8'h04, 8'h04, 8'h1F, 8'h04, 8'h04};   //4
		mem[8] = {8'hE0, 8'hF8, 8'h48, 8'h48, 8'h48, 8'h48, 8'h88, 8'h08};
		mem[9] = {8'h08, 8'h10, 8'h10, 8'h10, 8'h10, 8'h10, 8'h08, 8'h07};   //5
		mem[10] = {8'hE0, 8'hB0, 8'h98, 8'h88, 8'h88, 8'h88, 8'h08, 8'h10};
		mem[11] = {8'h07, 8'h09, 8'h10, 8'h10, 8'h10, 8'h10, 8'h09, 8'h0F};   //6
		mem[12] = {8'h08, 8'h08, 8'h08, 8'h08, 8'h88, 8'h68, 8'h18, 8'h08};
		mem[13] = {8'h00, 8'h00, 8'h18, 8'h07, 8'h00, 8'h00, 8'h00, 8'h00};   //7
		mem[14] = {8'h70, 8'hD8, 8'h88, 8'h88, 8'h88, 8'h88, 8'hD0, 8'h70};
		mem[15] = {8'h0F, 8'h09, 8'h10, 8'h10, 8'h10, 8'h10, 8'h09, 8'h0F};   //8
		mem[16] = {8'hF0, 8'h90, 8'h08, 8'h08, 8'h08, 8'h08, 8'h90, 8'hE0};
		mem[17] = {8'h08, 8'h10, 8'h11, 8'h11, 8'h11, 8'h19, 8'h0D, 8'h07};   //9
		mem[18] = {8'hE0, 8'h10, 8'h08, 8'h08, 8'h08, 8'h08, 8'h10, 8'hE0};
		mem[19] = {8'h07, 8'h08, 8'h10, 8'h10, 8'h10, 8'h10, 8'h08, 8'h07};   //0
		mem[20] = {8'h00, 8'h00, 8'h00, 8'h00, 8'hE0, 8'h00, 8'h00, 8'h00};
		mem[21] = {8'h00, 8'h01, 8'h01, 8'h01, 8'h07, 8'h01, 8'h01, 8'h01};   //+
		mem[22] =  {8'h00, 8'h80, 8'h80, 8'h80, 8'h80, 8'h80, 8'h80, 8'h00};
		mem[23] = {8'h00, 8'h00, 8'h00, 8'h00, 8'h00, 8'h00, 8'h00, 8'h00};   //-
		mem[24] = {8'h00, 8'h20, 8'h40, 8'h80, 8'h00, 8'h80, 8'h60, 8'h20};
		mem[25] = {8'h00, 8'h08, 8'h04, 8'h03, 8'h01, 8'h06, 8'h08, 8'h00};   //x
		mem[26]= {8'h00, 8'h00, 8'h00, 8'h80, 8'h60, 8'h10, 8'h0C, 8'h00}; 
		mem[27] = {8'h30, 8'h0C, 8'h02, 8'h01, 8'h00, 8'h00, 8'h00, 8'h00};   ///

	end
	
//clk_div = clk/CLK_DIV_PERIOD, 50% is high voltage
reg clk_div; 
reg[15:0] clk_cnt=0;

assign lcd_clk_out = clk_div;

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n) clk_cnt<=0;
	else begin
		if(clk_cnt==(CLK_DIV_PERIOD-1)) clk_cnt<=0;
		else clk_cnt<=clk_cnt+1; 
		if(clk_cnt<(CLK_DIV_PERIOD>>1)) clk_div<=0;
		else clk_div<=1;
	end
end

//divide clk_div 4 state, RISING and FALLING state is keeped one cycle of clk, like a pulse.
reg[1:0] clk_div_state=CLK_L;  
always@(posedge clk or negedge rst_n)
begin
	if(!rst_n) clk_div_state<=CLK_L;
    else 
		case(clk_div_state)
			CLK_L: begin
					if (clk_div) clk_div_state<=CLK_RISING_DEGE;  
					else clk_div_state<=CLK_L;
				end
			CLK_RISING_DEGE :clk_div_state<=CLK_H;  
			CLK_H:begin                 
					if (!clk_div) clk_div_state<=CLK_FALLING_DEGE;
					else clk_div_state<=CLK_H;
				end 
			CLK_FALLING_DEGE:clk_div_state<=CLK_L;  
			default;
		endcase
end

reg shift_flag = 0;
reg[6:0] x_reg;
reg[2:0] y_reg;   
reg[7:0] char_reg;
reg[7:0] cmd_reg;
reg[8:0] temp_cnt;
reg[7:0] data_reg; 
reg[7:0] data_state=IDLE; 
reg[7:0] data_state_back; 
reg[7:0] data_state_cnt=0;  
reg[3:0] shift_cnt=0; 
reg[25:0] delay_cnt=0;  
//Finite State Machine, 
always@(posedge clk or negedge rst_n)      
begin
	if(!rst_n) 
		begin 
			data_state<=IDLE;
			data_state_cnt<=0;
			shift_flag <= 0;
			lcd_cs_n_out<=HIGH;
		end
    else
		case (data_state)
			IDLE: begin
					lcd_cs_n_out<=HIGH;
					data_state_cnt<=data_state_cnt+1;
					case(data_state_cnt)
						0: lcd_rst_n_out <= 0;
						1: data_state<=DELAY;
						2: lcd_rst_n_out <= 1;
						3: data_state<=DELAY;
						//display initial
						4: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'he2;end
						5: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h2c;end
						6: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h2e;end
						7: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h2f;end
						8: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h20;end
						9: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h81;end
					   10: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h15;end
					   11: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'ha3;end
					   12: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'hc0;end
					   13: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'ha1;end
					   14: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h40;end
					   15: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'ha4;end
					   16: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'haf;end
					   
						//clear display
					   17: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'hb0;end
					   18: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h10;end
					   19: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h00;end
					   20: begin data_state<=CLEAR; data_state_back<=CLEAR; end
					   21: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'hb1;end
					   22: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h10;end
					   23: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h00;end
					   24: begin data_state<=CLEAR; data_state_back<=CLEAR; end
					   25: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'hb2;end
					   26: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h10;end
					   27: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h00;end
					   28: begin data_state<=CLEAR; data_state_back<=CLEAR;end
					   29: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'hb3;end
					   30: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h10;end
					   31: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h00;end
					   32: begin data_state<=CLEAR; data_state_back<=CLEAR;end
					   
						//set start point
					   33: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'hb0; end
					   34: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h10; end
					   35: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h03; end
						
						//wite the word
					   36: begin data_state<=DISPLAY;data_state_back<=DISPLAY;char_reg<=0; end
					   
					   37: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'hb1; end
					   38: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h10; end
					   39: begin data_state<=SETCMD;data_state_back<=IDLE;cmd_reg<=8'h03; end
					   //wite the word
					   40: begin data_state<=DISPLAY;data_state_back<=DISPLAY;char_reg<=1; end
											   
					   41: begin data_state_cnt<=41;end
						default;
					endcase
				end
				
			SHIFT: begin
					if(!shift_flag)
						begin
							if (clk_div_state==CLK_FALLING_DEGE)  
								begin
									if (shift_cnt==8)  
										begin
											shift_cnt<=0;
											data_state<=data_state_back;
										end
									else begin
											lcd_cs_n_out<=LOW;
											lcd_data_out<=data_reg[7];   
											shift_flag <= 1;
										end
								end
						end
					else
						begin
							if (clk_div_state==CLK_RISING_DEGE)   
								begin  
									data_reg<={data_reg[6:0], data_reg[7]};  
									shift_cnt<=shift_cnt+1;
									shift_flag <= 0;
								end
						end
				end
			SETCMD: begin
			
						lcd_cs_n_out<=HIGH;
						lcd_dc_out<=CMD;
						data_reg <= cmd_reg;
						data_state<=SHIFT;
					end
			
			DISPLAY: begin
						temp_cnt<=temp_cnt+1;
						lcd_cs_n_out<=HIGH;
						lcd_dc_out<=DATA;
						if (temp_cnt==8) begin
								data_state<=IDLE;
								temp_cnt<=0; 
						end else begin
								temp <= mem[char_reg];
								case (temp_cnt)
									0 :  data_reg<=temp[63:56];
									1 :  data_reg<=temp[55:48];
									2 :  data_reg<=temp[47:40];
									3 :  data_reg<=temp[39:32];
									4 :  data_reg<=temp[31:24];
									5 :  data_reg<=temp[23:16];
									6 :  data_reg<=temp[15:8];
									7 :  data_reg<=temp[7:0];
									default;
								endcase
								data_state<=SHIFT;
							end
					end
			
			CLEAR: begin            
					data_reg<=8'h00;
					temp_cnt<=temp_cnt+1;
					lcd_cs_n_out<=HIGH;
					lcd_dc_out<=DATA;
					if (temp_cnt>=132) 
						begin
							temp_cnt<=0;
							data_state<=IDLE;
						end
					else data_state<=SHIFT;
				end
			
			DELAY: begin
					if(delay_cnt==DELAY_PERIOD)
						begin
							data_state<=IDLE; 
							delay_cnt<=0;
						end
					else delay_cnt<=delay_cnt+1;
				end
					   
			default;
		endcase
end

endmodule
